/* Copyright (c) 2010 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* Publishing API.
 *
 * Provides useful methods and functions relating to publishing.
 */

if (typeof(ko)=='undefined') {
    var ko = {};
}

if (typeof(ko.publishing)=='undefined') {
    ko.publishing = {};
} else {
    dump("WARNING: ko.publishing was already loaded - reloading it.\n");
}

(function() {

    //var _bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
    //    .getService(Components.interfaces.nsIStringBundleService)
    //    .createBundle("chrome://komodo/locale/views.properties");
    var _observerSvc = Components.classes["@mozilla.org/observer-service;1"].
                    getService(Components.interfaces.nsIObserverService);
    var _publishingSvc = Components.classes["@activestate.com/koPublishingService;1"]
                                   .getService(Components.interfaces.koIPublishingService);

    var _publishing_configs = null;
    var _gUpdateMenusRequired = true;

    this.getConfigurations = function ko_pub_getConfigurations()
    {
        if (!_publishing_configs) {
            var settingsObj = {};
            _publishingSvc.getPublishingSettings({}, settingsObj);
            _publishing_configs = settingsObj.value;
        }
        return _publishing_configs;
    };

    this.saveConfigurations = function ko_pub_saveConfigurations(configs)
    {
        _publishingSvc.savePublishingSettings(configs.length, configs);
    };

    this.newConfiguration = function ko_pub_newConfiguration()
    {
        var args = {
            "type": "new"
        };
        ko.windowManager.openDialog('chrome://publishing/content/publish-dialog.xul',
                                    '_blank',
                                    'chrome,titlebar,resizable=yes',
                                    args);
    };

    this.synchronize = function ko_pub_synchronize(id)
    {
        var settings = null;
        var configs = ko.publishing.getConfigurations();
        for (var i=0; i < configs.length; i++) {
            if (configs[i].id == id) {
                settings = configs[i];
                break;
            }
        }
        var args = {
            "type": "synchronize",
            "settings": settings
        };
        ko.windowManager.openDialog('chrome://publishing/content/publish-dialog.xul',
                                    '_blank',
                                    'chrome,titlebar,resizable=yes',
                                    args);
    };

    this.onMenuPopupShowing = function ko_pub_onMenuPopupShowing()
    {
        if (_gUpdateMenusRequired) {
            var menupopup = document.getElementById("tools_publishing_menupopup");
            var separator = document.getElementById("tools_publishing_newconfig_separator");
            var prevSibling;
            var menuitem = separator.previousSibling;
            // Remove any existing menu items added previously.
            while (menuitem) {
                prevSibling = menuitem.previousSibling;
                if (menuitem.hasAttribute("dynamically_added_menuitem")) {
                    menupopup.removeChild(menuitem);
                }
                menuitem = prevSibling;
            }
            var attributes;
            var configs = this.getConfigurations();
            for (var i=0; i < configs.length; i++) {
                attributes = {
                    "label": configs[i].name,
                    "oncommand": "ko.publishing.synchronize('" + configs[i].id + "')",
                    "dynamically_added_menuitem": "true"
                }
                menuitem = xtk.domutils.newElement("menuitem", attributes);
                menupopup.insertBefore(menuitem, separator);
            }
            _gUpdateMenusRequired = false;
        }
    };

    this.onPlacesPopupShowing = function ko_pub_onPlacesPopupShowing()
    {
        var diffMenuitem = document.getElementById('placesContextMenu_publishingDiffFile');
        var koFileExes = ko.places.manager.getSelectedFiles();
        for (var i=0; i < koFileExes.length; i++) {
            if (koFileExes[i].publishingStatus) {
                diffMenuitem.removeAttribute("disabled");
                return;
            }
        }
        diffMenuitem.setAttribute("disabled", "true");
    };

        /**
         * Return the first publishing settings that match the given URI.
         * 
         * @param uri {string} - The file uri to match.
         * @returns {Components.interfaces.koIPublishingSettings}
         */
    this.getSettingsForUri = function ko_pub_getSettingsForUri(uri) {
        var configs = ko.publishing.getConfigurations();
        for (var i=0; i < configs.length; i++) {
            if (configs[i].matchesUri(uri)) {
                return configs[i];
            }
        }
        return null;
    };

        /**
         * Launches the diff dialog showing the file changes between the uris
         * and their corresponding local/remote publishing uris.
         *
         * @param uris {array} - The list of local or remote publishing uris.
         */
    this.diffURIs = function ko_pub_diffCurrentFile(uris) {
        var diffSvc = Components.classes["@activestate.com/koDiffService;1"].
                        getService(Components.interfaces.koIDiffService);
        var uri;
        var left_uris = [];
        var right_uris = [];

        for (var i=0; i < uris.length; i++) {
            uri = uris[i];
            var config = ko.publishing.getSettingsForUri(uri);
            if (!config) {
                continue;
            }
    
            var leftURI;
            var rightURI = uri;
    
            if (rightURI.substr(0, 5) == "file:") {
                // It's the local URI, work out the remote uri.
                leftURI = config.remote_uri + rightURI.substr(config.local_uri.length);
            } else {
                leftURI = config.local_uri + rightURI.substr(config.remote_uri.length);
            }
            left_uris.push(leftURI);
            right_uris.push(rightURI);
        }
        if (!left_uris) {
            ko.statusBar.AddMessage("No publishing configs found for any of: " + uris,
                                    "publishing", 5000, true, false);
            return;
        }

        var diffContent = diffSvc.diffMultipleURIs(left_uris, left_uris.length,
                                                   right_uris, right_uris.length);
        var title = "Publishing Diff";
        ko.launch.diff(diffContent, title);
    }

    /************************************************************
     *         Controller for the publishing commands           *
     ************************************************************/

    function PublishingController() {
    }
    PublishingController.prototype = new Controller();
    PublishingController.prototype.constructor = PublishingController;

    // Diff the current file.
    PublishingController.prototype.is_cmd_publishingDiffCurrentFile_enabled = function () {
        var view = ko.views.manager.currentView;
        var file = (view && view.koDoc && view.koDoc.file) || null;
        return file && file.publishingStatus;
    };
    PublishingController.prototype.do_cmd_publishingDiffCurrentFile = function () {
        var view = ko.views.manager.currentView;
        var file = (view && view.koDoc && view.koDoc.file) || null;
        if (!file) {
            ko.statusBar.AddMessage("The current view does not reference a file",
                                    "publishing", 5000, true, false);
            return;
        }
        ko.publishing.diffURIs([file.URI]);
    };

    // Push the current file.
    PublishingController.prototype.is_cmd_publishingPushCurrentFile_enabled = function () {
        var view = ko.views.manager.currentView;
        var file = (view && view.koDoc && view.koDoc.file) || null;
        return file && file.publishingStatus;
    };
    PublishingController.prototype.do_cmd_publishingPushCurrentFile = function () {
        ko.publishing.pushCurrentFile();
    };

    // Create new configuration.
    PublishingController.prototype.is_cmd_publishingNewConfiguration_enabled = function () {
        return true;
    };
    PublishingController.prototype.do_cmd_publishingNewConfiguration = function () {
        ko.publishing.newConfiguration();
    };

    // Diff the places file(s).
    PublishingController.prototype.is_cmd_publishingPlacesDiffFile_enabled = function () {
        return true;
    };
    PublishingController.prototype.do_cmd_publishingPlacesDiffFile = function () {
        var uris = ko.places.manager.getSelectedUris();
        if (!uris.length) {
            return;
        }
        ko.publishing.diffURIs(uris);
    };

    // Controller instance on the current window to handle profiler commands.
    var pubController = new PublishingController();
    window.controllers.appendController(pubController);

    /************************************************************
     *         Observer for catching configuration changes      *
     ************************************************************/

    this.observe = function ko_pub_observer(subject, topic, data) {
        if (topic == "publishing_configurations_changed") {
            _publishing_configs = null;
            _gUpdateMenusRequired = true;
        }
    };

    _observerSvc.addObserver(this, "publishing_configurations_changed", false);

}).apply(ko.publishing);
