/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */


var wizardManager = null;
var wd_info = null;
var _gOs;
var _gPhpInfo;
var _gPhpBuildInfoObj;
var _gOsPath;
var _gInfoSvc;
var log = ko.logging.getLogger("PHP Wizard");
//log.setLevel(ko.logging.LOG_DEBUG);
var _gFindingInterps = false;
var existingIni=null;
var currentDir=null;
var _bundle;


//---- interface routines for wsSetupWizard.xul

function onWizardLoad()
{
    // set wizard data for the panels
    wd_info = window.arguments[0];
    _gOs = Components.classes["@activestate.com/koOs;1"].createInstance();
    try {
        _gPhpInfo = Components.classes["@activestate.com/koPHPInfoInstance;1"].createInstance();
        if (_gPhpInfo.executablePath) {
            _gPhpBuildInfoObj = (Components.classes["@mozilla.org/dom/json;1"].
                                 createInstance(Components.interfaces.nsIJSON).
                                 decode(_gPhpInfo.phpBuildInfo));
        }
    } catch(ex) {
        log.exception("Wizard initialization failed: " + ex);
    }
    _gOsPath = Components.classes["@activestate.com/koOsPath;1"].getService();
    _gInfoSvc = Components.classes["@activestate.com/koInfoService;1"].getService();
    _bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
        .getService(Components.interfaces.nsIStringBundleService)
        .createBundle("chrome://komodo/locale/wizards.properties");

}

function onWizardCancel()
{
    // First, ensure that the user wants to cancel.
    var wantToCancel = ko.dialogs.yesNo(_bundle.GetStringFromName("cancelInstallingPHPDebuggerSupport.prompt"),
                                        "No");    // default answer

    // Second, go to the "Warning" panel.
    if (wantToCancel == "Yes") {
        wd_info.ok = false;
        return true;
    }
    return false;
}


function _chooseExecutable_InterpPathToInstallPath(interpreterPath)
{
    var installationPath = null;
    if (interpreterPath) {
        if (_gInfoSvc.platform.substring(0,3) == "win") {
            installationPath = _gOsPath.dirname(interpreterPath);
        } else {
            installationPath = _gOsPath.dirname(_gOsPath.dirname(
                interpreterPath));
        }
    }
    return installationPath;
}


function _chooseExecutable_UpdateInstallation(installation)
{
    // Persist.
    wd_info.interpreter = installation;

    // Display the installation selection.
    var installWidget = document.getElementById("installation");
    if (installation == null) {
        installWidget.value = "";
    } else {
        _gPhpInfo.executablePath = installWidget.value = installation;
        var phpDir = _gOsPath.dirname(installation);
        var iniPath = _gOsPath.join(phpDir, "php.ini");
        if (_gOsPath.exists(iniPath)) {
            _ini_UpdateExistingIni(iniPath);
        }
        _ini_UpdateDebuggerIniDir(phpDir);
    }
}



//---- Wizard Infrastructure interface routines

function chooseExecutable_validate()
{
    if (!wd_info.interpreter) {
        ko.dialogs.alert("Please specify the location for a PHP interpreter.");
    } else if (!_gOsPath.exists(wd_info.interpreter)) {
        ko.dialogs.alert("Komodo can't find the specified file: "
                         + wd_info.interpreter
                         + ".");
    } else {
        _gPhpInfo.executablePath = wd_info.interpreter;
        _gPhpBuildInfoObj = (Components.classes["@mozilla.org/dom/json;1"].
                             createInstance(Components.interfaces.nsIJSON).
                             decode(_gPhpInfo.phpBuildInfo));
        if (!_gPhpInfo.valid_version) {
            ko.dialogs.alert(_bundle.formatStringFromName("versionNotSupported.template",
                                                          [_gPhpInfo.version], 1));
        } else {
            return true;
        }
    }
    return false;
}


//---- interface for wsSetup_ChooseInstallation.xul

function chooseExecutable_onPageShow()
{
    // setup "avail-install-list"
    var listWidget = document.getElementById("avail-install-list");
    // - remove any existing items and add a "finding..." one
    _gFindingInterps = true;
    while (listWidget.hasChildNodes()) {
        listWidget.removeChild(listWidget.childNodes[0]);
    }
    listWidget.appendItem(_bundle.GetStringFromName("findingAvailablePHPInstallations"));
    // - get a list of installations
    var sysUtils = Components.classes['@activestate.com/koSysUtils;1'].
                   getService(Components.interfaces.koISysUtils);

    // get a list of installed PHP interpreters
    var numFound = new Object();
    var availInterps = _gPhpInfo.FindInstallationExecutables(numFound);

    // - populate the tree listing them
    listWidget.removeChild(listWidget.childNodes[0]); // "Finding..."
    if (availInterps.length != 0) {
        for (var i = 0; i < availInterps.length; i++) {
            listWidget.appendItem(availInterps[i]);
        }
    }
    _gFindingInterps = false;

    // If there is no installation setting or a new language was
    // selected then pick a reasonable default.
    if (!wd_info.interpreter) {
        wd_info.interpreter = _gPhpInfo.executablePath;
    }
    _chooseExecutable_UpdateInstallation(wd_info.interpreter);
}


function chooseExecutable_UpdateInstallation_OnSelection()
{
    // User selected one of the "found installations".
    if (! _gFindingInterps) {
        // Load the default installation textfield with the selection
        var listbox = document.getElementById("avail-install-list");
        if (listbox.selectedItem) 
            _chooseExecutable_UpdateInstallation(listbox.selectedItem.getAttribute('label'));
    }
}


function chooseExecutable_UpdateInstallation_OnKey()
{
    // User typed into the installation field.
    // - Unselect any tree item.
    try {
        var listbox = document.getElementById("avail-install-list");
        listbox.clearSelection();
    
        // - Persist the value.
        var installWidget = document.getElementById("installation");
        _chooseExecutable_UpdateInstallation(installWidget.value);
    } catch(e) { log.exception(e);}
}


function chooseExecutable_UpdateInstallation_OnBrowse()
{
    try {
        var current = document.getElementById('installation').value;
        var php = ko.filepicker.browseForExeFile(null, current);
        if (php != null) {
            _chooseExecutable_UpdateInstallation(php);
        }
    } catch(e) {
        log.exception(e);
    }
}

function ini_validate()
{
    wd_info.existingIni = document.getElementById("existingini").value;
    //TODO "ini" name is too vague -- this is actually the path to the
    // directory we're copying the revised php.ini file into.
    // I prefer "targetIniDir"
    wd_info.ini = document.getElementById("debuginidir").value;
    try {
        if (!wd_info.existingIni || !wd_info.ini) {
            ko.dialogs.alert(_bundle.GetStringFromName("selectIniFileAndDestination.prompt"));
            return false;
        }
        if (!_gOsPath.isfile(wd_info.existingIni)) {
            ko.dialogs.alert(_bundle.GetStringFromName("specifiedTargetIniDirNotValid.prompt"));
            return false;
        }
        if (_gOsPath.isfile(wd_info.ini)) {
            ko.dialogs.alert(_bundle.GetStringFromName("targetIniDestinationNotDirectory.prompt"));
            return false;
        }
        if (!_gOsPath.isdir(wd_info.ini)) {
            ko.dialogs.alert(_bundle.GetStringFromName("specifiedTargetIniDirDoesntExist.prompt"));
            return false;
        }
        if (!_gOs.saneAccessParent(wd_info.ini, Components.interfaces.koIOs.W_OK)) {
            ko.dialogs.alert(_bundle.GetStringFromName("noAccessToSpecifiedTargetIniDir.prompt"));
            return false;
        }
        return true;
    } catch(e) {
        log.exception(e);
    }
    return false;
}

function _ini_UpdateExistingIni(ini)
{
    // Persist.
    wd_info.existingIni = ini;

    // Display the installation selection.
    document.getElementById("existingini").value = ini;
}

function _ini_UpdateDebuggerIniDir(dir)
{
    // Persist.
    //TODO s{ini}{targetIniDir}
    wd_info.ini = _gOsPath.realpath(dir);

    // Display the installation selection.
    document.getElementById("debuginidir").value = dir;

    _extdir_UpdateExtensionDir(dir);
}

function ini_onPageShow()
{
    var selectedPHP = document.getElementById("installation").value;
    var selectedPHP_INI = null;
    if (selectedPHP) {
        selectedPHP_INI = _gOsPath.join(_gOsPath.dirname(selectedPHP),
                                        "php.ini");
        if (!_gOsPath.exists(selectedPHP_INI)) {
            selectedPHP_INI = null;
        }
    }

    if (document.getElementById("existingini").value.length == 0) {
        _ini_UpdateExistingIni(selectedPHP_INI || _gPhpInfo.cfg_file_path);
    }

    if (document.getElementById("debuginidir").value.length == 0) {
        var ini = _gOsPath.join(_gOsPath.dirname(wd_info.interpreter),'debug');
        if (_gOsPath.exists(ini) && wd_info.ini) {
            ini = wd_info.ini;
        }
        _ini_UpdateDebuggerIniDir(ini);
    }
}

function ini_UpdateExistingIni_OnBrowse()
{
    var defaultDir=null, defaultFilename=null;
    if (existingIni) {
        defaultDir = null;
        defaultFilename = existingIni;
    } else {
        defaultDir = null; //XXX Could do better to suggest dir here.
        defaultFilename = "php.ini";
    }
    var ini = ko.filepicker.browseForFile(
                defaultDir,
                defaultFilename,// default filename
                _bundle.GetStringFromName("chooseSourceINIFile.prompt"), //title
                "INI", // default filter
                ["INI", "All"]); // filters
    if (ini != null) {
        _ini_UpdateExistingIni(ini);
    }
}

function ini_UpdateDebuggerIni_OnBrowse()
{
    var dir = document.getElementById("debuginidir").value;
    // bug77856 - validate the default dir field
    if (!dir || !_gOsPath.isdir(dir)) {
        if (!dir) {
            dir = existingIni && _gOsPath.dirname(existingIni);
        } else {
            dir = _gOsPath.dirname(dir);
        }
        // One last sanity check, or the file-picker will
        // give a confusing error message, using file:/// URIs.
        if (!dir || !_gOsPath.isdir(dir)) {
            try {
                dir = Components.classes['@activestate.com/koUserEnviron;1'].
                      getService(Components.interfaces.koIUserEnviron).get("HOME");
            } catch(ex) {
                dir = "";  // define *something*
            }
        }
    }
    
    var inipath = ko.filepicker.getFolder(dir);
    if (inipath != null) {
        _ini_UpdateDebuggerIniDir(inipath);
    }
}

function ini_UpdateDebuggerIni_OnKey(textbox)
{
    var text = textbox.value;
    if (text.length > 0) {
        _ini_UpdateDebuggerIniDir(text);
    }
}

function extdir_validate()
{
    try {
        wd_info.extensionDir = document.getElementById("target_extensiondir").value;
        if (!wd_info.extensionDir) {
            ko.dialogs.alert(_bundle.GetStringFromName("noExtensionDirectoryProvided.prompt"));
            return false;
        }
        if (!_gOs.saneAccessParent(wd_info.extensionDir, Components.interfaces.koIOs.W_OK)) {
            ko.dialogs.alert(_bundle.GetStringFromName("noAccessToSpecifiedTargetExtensionDir.prompt"));
            return false;
        }
    } catch(e) {
        log.exception(e);
    }
    return true;
}

function _extdir_UpdateExistingDir(dir)
{
    // Persist.
    currentDir = dir;
    if (!wd_info.extensionDir)
        wd_info.extensionDir = dir;

    // Display the installation selection.
    document.getElementById("existingextdir").value = dir;
}

function _extdir_UpdateExtensionDir(dir)
{
    // Persist.
    wd_info.extensionDir = _gOsPath.realpath(dir);

    // Display the installation selection.
    document.getElementById("target_extensiondir").value = dir;
}

function extdir_onPageShow()
{
    // use the ini we're going to copy
    _gPhpInfo.cfg_file_path = _gOsPath.dirname(wd_info.ini);
    _extdir_UpdateExistingDir(_gPhpInfo.extension_dir);
    
    if (document.getElementById("target_extensiondir").value.length == 0) {
        var dir = wd_info.extensionDir;
        // an unconfigured ini file with have ./
        if (!dir || dir == './') {
            dir = _gOsPath.join(_gOsPath.dirname(wd_info.interpreter),'extensions');
        }
        _extdir_UpdateExtensionDir(dir);
    }
}

function extdir_UpdateDir_OnBrowse()
{
    var dir = document.getElementById("target_extensiondir").value;
    if (!dir) {
        dir = wd_info.ini;
    }
    var extensionDir = ko.filepicker.getFolder(dir)
    if (extensionDir != null) {
        _extdir_UpdateExtensionDir(extensionDir);
    }
}


function install_onPageShow()
{
    // Describe the installation procedure.
    var actionWidget = document.getElementById("php-executable");
    var description = _bundle.formatStringFromName('phpExecutable.format',
                                                   [parent.wd_info.interpreter], 1);
    var textNode = document.createTextNode(description);
    while (actionWidget.firstChild) {
        actionWidget.removeChild(actionWidget.firstChild)
    }
    actionWidget.appendChild(textNode);

    actionWidget = document.getElementById("php-inipath");
    while (actionWidget.firstChild) {
        actionWidget.removeChild(actionWidget.firstChild)
    }
    description = _bundle.formatStringFromName('phpINIPath.format',
                                               [parent.wd_info.ini], 1);
    textNode = document.createTextNode(description);
    actionWidget.appendChild(textNode);

    actionWidget = document.getElementById("php-extensions");
    while (actionWidget.firstChild) {
        actionWidget.removeChild(actionWidget.firstChild)
    }
    description = _bundle.formatStringFromName('phpINIPath.format',
                                               [parent.wd_info.extensionDir], 1);
    textNode = document.createTextNode(description);
    actionWidget.appendChild(textNode);
}


function _installing_Install()
{
    var msg=null;
    // XXX If this is a long process then should really add a progress
    //     meter, or perhaps stream in the PyPPM output as it comes.
    window.setCursor("spinning");
    // do it now
    try {
        var configure = Components.classes["@activestate.com/koPHPConfigurator;1"].
                createInstance(Components.interfaces.koIPHPConfigurator);

        msg = configure.configureDebugger(
                                wd_info.interpreter,
                                wd_info.existingIni,
                                wd_info.ini,
                                wd_info.extensionDir
                                )
    } catch(e) {
        log.exception(e);
        msg = _bundle.GetStringFromName("internalError.prompt");
    }
    if (msg.indexOf('PHP is configured, but was unable to load the debugger extension') == 0) {
        var msg2 = ("The xdebug extension was found for PHP, but couldn't be loaded.  Here's the pertinent build info for PHP (installed at"
                    + _gPhpInfo.executablePath
                    + "): \n\n");
        for (var p in _gPhpBuildInfoObj) {
            msg2 += p + ":" + _gPhpBuildInfoObj[p] + "\n";
        }
        msg += "\n\n" + msg2;
    }
    window.setCursor("auto");

    // Determine if successful.
    wd_info.msg = msg;
    if (!msg) {
        wd_info.ok = true;
    } else {
        wd_info.ok = false;
    }
}



//---- interface for wsSetup_Installing.xul

function installing_onPageShow()
{
    // <description id="please-wait-msg"/>
    var pleaseWaitMsg = document.getElementById("please-wait-msg");
    var text = _bundle.GetStringFromName("pleaseWait.prompt");
    while (pleaseWaitMsg.firstChild) {
        pleaseWaitMsg.removeChild(pleaseWaitMsg.firstChild)
    }
    pleaseWaitMsg.appendChild(document.createTextNode(text));

    // Setup for WS.
    var wizard = document.getElementById("php-install-wizard-window");
    wizard.canAdvance = false;
    wizard.canRewind = false;
    try {
        _installing_Install();
    } catch(e) {
        log.exception(e);
        wd_info.msg = _bundle.GetStringFromName("internalError.prompt");
        wd_info.ok = false;
    }
    wizard.canAdvance = true;
    wizard.advance(null);
}


function finished_onPageShow()
{
    var wizard = document.getElementById("php-install-wizard-window");
    wizard.getButton('cancel').setAttribute('collapsed','true');
    wizard.getButton('back').setAttribute('collapsed','true');
    wizard.getButton('next').setAttribute('collapsed','true');
    var donemsg = document.getElementById("finish-msg");
    var text;
    if (parent.wd_info.ok) {
        text = _bundle.GetStringFromName("configurationSucceeded.prompt");
    } else {
        text = _bundle.formatStringFromName("configurationFailed.template",
                                            [parent.wd_info.msg], 1);
    }
    while (donemsg.firstChild) {
        donemsg.removeChild(donemsg.firstChild)
    }

    var textNodes = text.split(/\n/);
    var numTextNodes = textNodes.length;
    if (numTextNodes == 1) {
        donemsg.appendChild(document.createTextNode(text));
    } else {
        try {
            var parentNode = donemsg.parentNode;
            donemsg.appendChild(document.createTextNode(textNodes[0]));
            for (var i = 1; i < numTextNodes; ++i) {
                var tn = textNodes[i];
                var descr = document.createElement("description");
                parentNode.appendChild(descr);
                descr.appendChild(document.createTextNode(tn));
            }
        } catch(e) {
            log.exception(e);
        }
    }
}
